@@ -46,6 +46,7 @@
 #include "gdata-documents-presentation.h"
 #include "gdata-service.h"
 #include "gdata-private.h"
+#include "gdata-upload-stream.h"
 
 GQuark
 gdata_documents_service_error_quark (void)
@@ -57,9 +58,8 @@ static void gdata_documents_service_dispose (GObject *object);
 static void gdata_documents_service_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void notify_authenticated_cb (GObject *service, GParamSpec *pspec, GObject *self);
 static void notify_proxy_uri_cb (GObject *service, GParamSpec *pspec, GObject *self);
-GDataDocumentsEntry *upload_update_document (GDataDocumentsService *self, GDataDocumentsEntry *document, GFile *document_file,
-					     SoupMessage *message, guint good_status_code,
-					     GCancellable *cancellable, GError **error);
+static GDataDocumentsEntry *upload_update_document (GDataDocumentsService *self, GDataDocumentsEntry *document, GFile *document_file,
+						    const gchar *method, const gchar *upload_uri, GCancellable *cancellable, GError **error);
 
 struct _GDataDocumentsServicePrivate {
 	GDataService *spreadsheet_service;
@@ -182,6 +182,9 @@ gdata_documents_service_query_documents (GDataDocumentsService *self, GDataDocum
 					 GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
 					 GError **error)
 {
+	GDataFeed *feed;
+	gchar *request_uri;
+
 	/* Ensure we're authenticated first */
 	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
@@ -189,8 +192,83 @@ gdata_documents_service_query_documents (GDataDocumentsService *self, GDataDocum
 		return NULL;
 	}
 
-	return GDATA_DOCUMENTS_FEED (gdata_service_query (GDATA_SERVICE (self), "http://docs.google.com/feeds/documents/private/full", GDATA_QUERY (query),
-				     GDATA_TYPE_DOCUMENTS_ENTRY, cancellable, progress_callback, progress_user_data, error));
+	/* If we want to query for documents contained in a folder, the URI is different */
+	if (query != NULL && gdata_documents_query_get_folder_id (query) != NULL)
+		request_uri = g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/folders/private/full", NULL);
+	else
+		request_uri = g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/documents/private/full", NULL);
+
+	feed = gdata_service_query (GDATA_SERVICE (self), request_uri, GDATA_QUERY (query), GDATA_TYPE_DOCUMENTS_ENTRY, cancellable,
+	                            progress_callback, progress_user_data, error);
+	g_free (request_uri);
+
+	return GDATA_DOCUMENTS_FEED (feed);
+}
+
+/**
+ * gdata_documents_service_query_single_document:
+ * @self: a #GDataDocumentsService
+ * @document_type: the expected #GType of the queried entry
+ * @document_id: the document ID of the queried document
+ * @cancellable: a #GCancellable, or %NULL
+ * @error: a #GError, or %NULL
+ *
+ * Retrieves information about a single document with the given document ID.
+ *
+ * @document_type should be the expected type of the document to be returned. e.g. %GDATA_TYPE_DOCUMENTS_SPREADSHEET if you're querying
+ * for a spreadsheet.
+ *
+ * @document_id should be the ID of the document as returned by gdata_document_entry_get_document_id().
+ *
+ * Parameters and errors are as for gdata_service_query().
+ *
+ * Return value: a #GDataDocumentsEntry, or %NULL; unref with g_object_unref()
+ *
+ * Since: 0.5.0
+ **/
+GDataDocumentsEntry *
+gdata_documents_service_query_single_document (GDataDocumentsService *self, GType document_type, const gchar *document_id,
+					       GCancellable *cancellable, GError **error)
+{
+	GDataDocumentsEntry *document;
+	SoupMessage *message;
+	GDataDocumentsQuery *query;
+	gchar *resource_id, *request_uri;
+
+	g_return_val_if_fail (GDATA_IS_DOCUMENTS_SERVICE (self), NULL);
+	g_return_val_if_fail (document_id != NULL, NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+	if (document_type == GDATA_TYPE_DOCUMENTS_FOLDER)
+		resource_id = g_strconcat ("folder:", document_id, NULL);
+	else if (document_type == GDATA_TYPE_DOCUMENTS_SPREADSHEET)
+		resource_id = g_strconcat ("spreadsheet:", document_id, NULL);
+	else if (document_type == GDATA_TYPE_DOCUMENTS_TEXT)
+		resource_id = g_strconcat ("document:", document_id, NULL);
+	else if (document_type == GDATA_TYPE_DOCUMENTS_PRESENTATION)
+		resource_id = g_strconcat ("presentation:", document_id, NULL);
+	else
+		g_assert_not_reached ();
+
+	query = gdata_documents_query_new (NULL);
+	gdata_query_set_entry_id (GDATA_QUERY (query), resource_id);
+	g_free (resource_id);
+
+	request_uri = g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/documents/private/full", NULL);
+	message = _gdata_service_query (GDATA_SERVICE (self), request_uri, GDATA_QUERY (query), cancellable, NULL, NULL, error);
+	g_free (request_uri);
+	g_object_unref (query);
+
+	if (message == NULL)
+		return NULL;
+
+	g_assert (message->response_body->data != NULL);
+	document = GDATA_DOCUMENTS_ENTRY (gdata_parsable_new_from_xml (document_type, message->response_body->data,
+								       message->response_body->length, error));
+	g_object_unref (message);
+
+	return document;
 }
 
 /**
@@ -216,6 +294,8 @@ gdata_documents_service_query_documents_async (GDataDocumentsService *self, GDat
 					       GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
 					       GAsyncReadyCallback callback, gpointer user_data)
 {
+	gchar *request_uri;
+
 	/* Ensure we're authenticated first */
 	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
 		g_simple_async_report_error_in_idle (G_OBJECT (self), callback, user_data,
@@ -224,14 +304,16 @@ gdata_documents_service_query_documents_async (GDataDocumentsService *self, GDat
 		return;
 	}
 
-	gdata_service_query_async (GDATA_SERVICE (self), "http://docs.google.com/feeds/documents/private/full", GDATA_QUERY (query),
-				   GDATA_TYPE_DOCUMENTS_ENTRY, cancellable, progress_callback, progress_user_data, callback, user_data);
+	request_uri = g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/documents/private/full", NULL);
+	gdata_service_query_async (GDATA_SERVICE (self), request_uri, GDATA_QUERY (query), GDATA_TYPE_DOCUMENTS_ENTRY,
+	                           cancellable, progress_callback, progress_user_data, callback, user_data);
+	g_free (request_uri);
 }
 
-/**
- * To upload spreasheet documents, another token is needed since the service for it is "wise" as apposed to "writely" for other operations.
+/*
+ * To upload spreadsheet documents, another token is needed since the service for it is "wise" as apposed to "writely" for other operations.
  * This callback aims to authenticate to this service as a private property (@priv->spreadsheet_service) of #GDataDocumentsService.
- * */
+ */
 static void
 notify_authenticated_cb (GObject *service, GParamSpec *pspec, GObject *self)
 {
@@ -261,178 +343,99 @@ notify_proxy_uri_cb (GObject *service, GParamSpec *pspec, GObject *self)
 	gdata_service_set_proxy_uri (GDATA_DOCUMENTS_SERVICE (self)->priv->spreadsheet_service, proxy_uri);
 }
 
-GDataDocumentsEntry *
-upload_update_document (GDataDocumentsService *self, GDataDocumentsEntry *document, GFile *document_file, SoupMessage *message,
-			guint good_status_code, GCancellable *cancellable, GError **error)
+static GDataDocumentsEntry *
+upload_update_document (GDataDocumentsService *self, GDataDocumentsEntry *document, GFile *document_file, const gchar *method,
+			const gchar *upload_uri, GCancellable *cancellable, GError **error)
 {
-	/* TODO: Async variant */
-	#define BOUNDARY_STRING "0003Z5W789RTE456KlemsnoZV"
+	GDataDocumentsEntry *new_entry;
+	GOutputStream *output_stream;
+	GInputStream *input_stream;
+	const gchar *slug = NULL, *content_type = NULL, *response_body;
+	gssize response_length;
+	GFileInfo *file_info = NULL;
+	GType new_document_type;
+	GError *child_error = NULL;
 
-	GDataServiceClass *klass;
-	GType new_document_type = GDATA_TYPE_DOCUMENTS_ENTRY; /* If the document type is wrong we get an error from the server anyway */
-	gchar *entry_xml, *second_chunk_header, *upload_data, *document_contents = NULL, *i;
-	const gchar *first_chunk_header, *footer;
-	GFileInfo *document_file_info = NULL;
-	guint status;
-	gsize content_length, first_chunk_header_length, second_chunk_header_length, entry_xml_length, document_length, footer_length;
-
-	g_return_val_if_fail (document != NULL || document_file != NULL, NULL);
-	g_return_val_if_fail (message != NULL, NULL);
-
-	/* Make sure subclasses set their headers */
-	klass = GDATA_SERVICE_GET_CLASS (self);
-	if (klass->append_query_headers != NULL)
-		klass->append_query_headers (GDATA_SERVICE (self), message);
-
-	/* Gets document file information */
+	/* Get some information about the file we're uploading */
 	if (document_file != NULL) {
-		/* Get the data early so we can calculate the content length */
-		if (g_file_load_contents (document_file, NULL, &document_contents, &document_length, NULL, error) == FALSE)
+		/* Get the slug and content type */
+		file_info = g_file_query_info (document_file, "standard::display-name,standard::content-type", G_FILE_QUERY_INFO_NONE, NULL, error);
+		if (file_info == NULL)
 			return NULL;
 
-		document_file_info = g_file_query_info (document_file, "standard::display-name,standard::content-type", G_FILE_QUERY_INFO_NONE, NULL, error);
-		if (document_file_info == NULL) {
-			g_free (document_contents);
-			return NULL;
-		}
+		slug = g_file_info_get_display_name (file_info);
+		content_type = g_file_info_get_content_type (file_info);
 
-		/* Check for cancellation */
-		if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE) {
-			g_free (document_contents);
-			g_object_unref (document_file_info);
-			return NULL;
-		}
-
-		/* Add document-upload--specific headers */
-		soup_message_headers_append (message->request_headers, "Slug", g_file_info_get_display_name (document_file_info));
-
-		if (document == NULL) {
-			const gchar *content_type = g_file_info_get_content_type (document_file_info);
-
-			/* Corrects a bug on spreadsheet content types handling
-			 * The content type for ODF spreasheets is "application/vnd.oasis.opendocument.spreadsheet" for my ODF spreadsheet;
-			 * but Google Documents' spreadsheet service is waiting for "application/x-vnd.oasis.opendocument.spreadsheet"
-			 * and nothing else.
-			 * Bug filed with Google: http://code.google.com/p/gdata-issues/issues/detail?id=1127 */
-			if (strcmp (content_type, "application/vnd.oasis.opendocument.spreadsheet") == 0)
-				content_type = "application/x-vnd.oasis.opendocument.spreadsheet";
-			soup_message_set_request (message, content_type, SOUP_MEMORY_TAKE, document_contents, document_length);
-
-			/* We get the document type of the document which is being uploaded */
-			if (strcmp (content_type, "application/x-vnd.oasis.opendocument.spreadsheet") == 0 ||
-			    strcmp (content_type, "text/tab-separated-values") == 0 ||
-			    strcmp (content_type, "application/x-vnd.oasis.opendocument.spreadsheet") == 0 ||
-			    strcmp (content_type, "application/vnd.ms-excel") == 0) {
-				new_document_type = GDATA_TYPE_DOCUMENTS_SPREADSHEET;
-			} else if (strcmp (content_type, "application/msword") == 0 ||
-				 strcmp (content_type, "application/vnd.oasis.opendocument.text") == 0 ||
-				 strcmp (content_type, "application/rtf") == 0 ||
-				 strcmp (content_type, "text/html") == 0 ||
-				 strcmp (content_type, "application/vnd.sun.xml.writer") == 0 ||
-				 strcmp (content_type, "text/plain") == 0) {
-				new_document_type = GDATA_TYPE_DOCUMENTS_TEXT;
-			} else if (strcmp (content_type, "application/vnd.ms-powerpoint") == 0) {
-				new_document_type = GDATA_TYPE_DOCUMENTS_PRESENTATION;
-			} else {
-				g_set_error_literal (error, GDATA_DOCUMENTS_SERVICE_ERROR, GDATA_DOCUMENTS_SERVICE_ERROR_INVALID_CONTENT_TYPE,
-						     _("The supplied document had an invalid content type."));
-				g_free (document_contents);
-				g_object_unref (document_file_info);
-				return NULL;
-			}
-		}
+		/* Corrects a bug on spreadsheet content types handling
+		 * The content type for ODF spreadsheets is "application/vnd.oasis.opendocument.spreadsheet" for my ODF spreadsheet;
+		 * but Google Documents' spreadsheet service is waiting for "application/x-vnd.oasis.opendocument.spreadsheet"
+		 * and nothing else.
+		 * Bug filed with Google: http://code.google.com/p/gdata-issues/issues/detail?id=1127 */
+		if (strcmp (content_type, "application/vnd.oasis.opendocument.spreadsheet") == 0)
+			content_type = "application/x-vnd.oasis.opendocument.spreadsheet";
 	}
 
-	/* Prepare XML file to upload metadata */
-	if (document != NULL) {
-		/* Test on the document entry */
-		new_document_type = G_OBJECT_TYPE (document);
-
-		/* Get the XML content */
-		entry_xml = gdata_parsable_get_xml (GDATA_PARSABLE (document));
-
-		/* Check for cancellation */
-		if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE) {
-			g_free (entry_xml);
-			g_free (document_contents);
-			if (document_file_info != NULL)
-				g_object_unref (document_file_info);
-			return NULL;
-		}
-
-		/* Prepare the data to upload containg both metadata and document content */
-		if (document_file != NULL) {
-			const gchar *content_type = g_file_info_get_content_type (document_file_info);
-
-			first_chunk_header = "--" BOUNDARY_STRING "\nContent-Type: application/atom+xml; charset=UTF-8\n\n<?xml version='1.0'?>";
-			/* Corrects a bug on spreadsheet content types handling (see above) */
-			if (strcmp (content_type, "application/vnd.oasis.opendocument.spreadsheet") == 0)
-				content_type = "application/x-vnd.oasis.opendocument.spreadsheet";
-
-			second_chunk_header = g_strdup_printf ("\n--" BOUNDARY_STRING "\nContent-Type: %s\n\n", content_type);
-			footer = "\n--" BOUNDARY_STRING "--";
-
-			first_chunk_header_length = strlen (first_chunk_header);
-			second_chunk_header_length = strlen (second_chunk_header);
-			entry_xml_length = strlen (entry_xml);
-			footer_length = strlen (footer);
-			content_length = first_chunk_header_length + entry_xml_length + second_chunk_header_length + document_length + footer_length;
-
-			/* Build the upload data */
-			upload_data = i = g_malloc (content_length);
-
-			memcpy (upload_data, first_chunk_header, first_chunk_header_length);
-			i += first_chunk_header_length;
-
-			memcpy (i, entry_xml, entry_xml_length);
-			i += entry_xml_length;
-			g_free (entry_xml);
-
-			memcpy (i, second_chunk_header, second_chunk_header_length);
-			g_free (second_chunk_header);
-			i += second_chunk_header_length;
-
-			memcpy (i, document_contents, document_length);
-			g_free (document_contents);
-			i += document_length;
-
-			memcpy (i, footer, footer_length);
-
-			/* Append the data */
-			soup_message_set_request (message, "multipart/related; boundary=" BOUNDARY_STRING, SOUP_MEMORY_TAKE, upload_data, content_length);
+	/* Determine the type of the document we're uploading */
+	if (document == NULL) {
+		/* We get the document type of the document which is being uploaded */
+		if (strcmp (content_type, "application/x-vnd.oasis.opendocument.spreadsheet") == 0 ||
+		    strcmp (content_type, "text/tab-separated-values") == 0 ||
+		    strcmp (content_type, "application/x-vnd.oasis.opendocument.spreadsheet") == 0 ||
+		    strcmp (content_type, "application/vnd.ms-excel") == 0) {
+			new_document_type = GDATA_TYPE_DOCUMENTS_SPREADSHEET;
+		} else if (strcmp (content_type, "application/msword") == 0 ||
+			 strcmp (content_type, "application/vnd.oasis.opendocument.text") == 0 ||
+			 strcmp (content_type, "application/rtf") == 0 ||
+			 strcmp (content_type, "text/html") == 0 ||
+			 strcmp (content_type, "application/vnd.sun.xml.writer") == 0 ||
+			 strcmp (content_type, "text/plain") == 0) {
+			new_document_type = GDATA_TYPE_DOCUMENTS_TEXT;
+		} else if (strcmp (content_type, "application/vnd.ms-powerpoint") == 0) {
+			new_document_type = GDATA_TYPE_DOCUMENTS_PRESENTATION;
 		} else {
-			/* Send only metadata */
-			upload_data = g_strconcat ("<?xml version='1.0' encoding='UTF-8'?>", entry_xml, NULL);
-			g_free (entry_xml);
-			soup_message_set_request (message, "application/atom+xml", SOUP_MEMORY_TAKE, upload_data, strlen (upload_data));
+			g_set_error_literal (error, GDATA_DOCUMENTS_SERVICE_ERROR, GDATA_DOCUMENTS_SERVICE_ERROR_INVALID_CONTENT_TYPE,
+					     _("The supplied document had an invalid content type."));
+			if (file_info != NULL)
+				g_object_unref (file_info);
+			return NULL;
 		}
-
-		if (document_file_info != NULL)
-			g_object_unref (document_file_info);
+	} else {
+		new_document_type = G_OBJECT_TYPE (document);
 	}
 
-	/* Send the message */
-	status = _gdata_service_send_message (GDATA_SERVICE (self), message, error);
-	if (status == SOUP_STATUS_NONE)
+	/* We need streaming file I/O: GDataUploadStream */
+	output_stream = gdata_upload_stream_new (GDATA_SERVICE (self), method, upload_uri, GDATA_ENTRY (document), slug, content_type);
+
+	if (file_info != NULL)
+		g_object_unref (file_info);
+	if (output_stream == NULL)
 		return NULL;
 
-	/* Check for cancellation */
-	if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE)
-		return NULL;
-
-	if (status != good_status_code) {
-		/* Error */
-		g_assert (klass->parse_error_response != NULL);
-		klass->parse_error_response (GDATA_SERVICE (self), GDATA_SERVICE_ERROR_WITH_INSERTION, status, message->reason_phrase, message->response_body->data,
-					     message->response_body->length, error);
+	/* Open the document file for reading and pipe it to the upload stream */
+	input_stream = G_INPUT_STREAM (g_file_read (document_file, cancellable, error));
+	if (input_stream == NULL) {
+		g_object_unref (output_stream);
 		return NULL;
 	}
 
-	/* Build the updated entry */
-	g_assert (message->response_body->data != NULL);
+	g_output_stream_splice (output_stream, input_stream, G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
+				cancellable, &child_error);
 
-	/* Parse the XML; create and return a new GDataEntry of the same type as @entry */
-	return GDATA_DOCUMENTS_ENTRY (gdata_parsable_new_from_xml (new_document_type, message->response_body->data, (gint) message->response_body->length, error));
+	g_object_unref (input_stream);
+	if (child_error != NULL) {
+		g_object_unref (output_stream);
+		g_propagate_error (error, child_error);
+		return NULL;
+	}
+
+	/* Get and parse the response from the server */
+	response_body = gdata_upload_stream_get_response (GDATA_UPLOAD_STREAM (output_stream), &response_length);
+	g_assert (response_body != NULL && response_length > 0);
+
+	new_entry = GDATA_DOCUMENTS_ENTRY (gdata_parsable_new_from_xml (new_document_type, response_body, (gint) response_length, error));
+	g_object_unref (output_stream);
+
+	return new_entry;
 }
 
 /**
@@ -466,8 +469,7 @@ gdata_documents_service_upload_document (GDataDocumentsService *self, GDataDocum
 					 GCancellable *cancellable, GError **error)
 {
 	GDataDocumentsEntry *new_document;
-	SoupMessage *message;
-	gchar *upload_uri, *tmp_str = NULL;
+	gchar *upload_uri;
 
 	g_return_val_if_fail (GDATA_IS_DOCUMENTS_SERVICE (self), NULL);
 	g_return_val_if_fail (document == NULL || GDATA_IS_DOCUMENTS_ENTRY (document), NULL);
@@ -488,20 +490,15 @@ gdata_documents_service_upload_document (GDataDocumentsService *self, GDataDocum
 		return NULL;
 	}
 
-	/* Get the upload URI */
-	if (folder != NULL) {
-		const gchar *folder_id = gdata_documents_entry_get_document_id (GDATA_DOCUMENTS_ENTRY (folder));
-		g_assert (folder_id != NULL);
-		upload_uri = tmp_str = g_strconcat ("http://docs.google.com/feeds/folders/private/full/folder%3A", folder_id, NULL);
+	upload_uri = gdata_documents_service_get_upload_uri (folder);
+
+	if (document_file == NULL) {
+		new_document = GDATA_DOCUMENTS_ENTRY (gdata_service_insert_entry (GDATA_SERVICE (self), upload_uri, GDATA_ENTRY (document),
+										  cancellable, error));
 	} else {
-		upload_uri = "http://docs.google.com/feeds/documents/private/full";
+		new_document = upload_update_document (self, document, document_file, SOUP_METHOD_POST, upload_uri, cancellable, error);
 	}
-
-	message = soup_message_new (SOUP_METHOD_POST, upload_uri);
-	g_free (tmp_str);
-
-	new_document = upload_update_document (self, document, document_file, message, 201, cancellable, error);
-	g_object_unref (message);
+	g_free (upload_uri);
 
 	return new_document;
 }
@@ -514,7 +511,8 @@ gdata_documents_service_upload_document (GDataDocumentsService *self, GDataDocum
  * @cancellable: optional #GCancellable object, or %NULL
  * @error: a #GError, or %NULL
  *
- * Update the document using the properties from @document and the document file pointed to by @document_file.
+ * Update the document using the properties from @document and the document file pointed to by @document_file. If the document file does not
+ * need to be changed, @document_file can be %NULL.
  *
  * If there is a problem reading @document_file, an error from g_file_load_contents() or g_file_query_info() will be returned. Other errors from
  * #GDataServiceError can be returned for other exceptional conditions, as determined by the server.
@@ -529,9 +527,7 @@ GDataDocumentsEntry *
 gdata_documents_service_update_document (GDataDocumentsService *self, GDataDocumentsEntry *document, GFile *document_file,
 					 GCancellable *cancellable, GError **error)
 {
-	GDataDocumentsEntry *updated_document;
-	SoupMessage *message;
-	GDataLink *update_uri;
+	GDataLink *update_link;
 
 	g_return_val_if_fail (GDATA_IS_DOCUMENTS_SERVICE (self), NULL);
 	g_return_val_if_fail (GDATA_IS_DOCUMENTS_ENTRY (document), NULL);
@@ -545,18 +541,12 @@ gdata_documents_service_update_document (GDataDocumentsService *self, GDataDocum
 	}
 
 	if (document_file == NULL)
-		update_uri = gdata_entry_look_up_link (GDATA_ENTRY (document), GDATA_LINK_EDIT);
-	else
-		update_uri = gdata_entry_look_up_link (GDATA_ENTRY (document), GDATA_LINK_EDIT_MEDIA);
-	g_assert (update_uri != NULL);
+		return GDATA_DOCUMENTS_ENTRY (gdata_service_update_entry (GDATA_SERVICE (self), GDATA_ENTRY (document), cancellable, error));
 
-	message = soup_message_new (SOUP_METHOD_PUT, gdata_link_get_uri (update_uri));
-	soup_message_headers_append (message->request_headers, "If-Match", gdata_entry_get_etag (GDATA_ENTRY (document)));
+	update_link = gdata_entry_look_up_link (GDATA_ENTRY (document), GDATA_LINK_EDIT_MEDIA);
+	g_assert (update_link != NULL);
 
-	updated_document = upload_update_document (self, document, document_file, message, 200, cancellable, error);
-	g_object_unref (message);
-
-	return updated_document;
+	return upload_update_document (self, document, document_file, SOUP_METHOD_PUT, gdata_link_get_uri (update_link), cancellable, error);
 }
 
 /**
@@ -600,7 +590,7 @@ gdata_documents_service_move_document_to_folder (GDataDocumentsService *self, GD
 
 	folder_id = gdata_documents_entry_get_document_id (GDATA_DOCUMENTS_ENTRY (folder));
 	g_assert (folder_id != NULL);
-	uri = g_strconcat ("http://docs.google.com/feeds/folders/private/full/folder%3A", folder_id, NULL);
+	uri = g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/folders/private/full/folder%3A", folder_id, NULL);
 
 	message = soup_message_new (SOUP_METHOD_POST, uri);
 	g_free (uri);
@@ -640,8 +630,8 @@ gdata_documents_service_move_document_to_folder (GDataDocumentsService *self, GD
 	if (status != 201) {
 		/* Error */
 		g_assert (klass->parse_error_response != NULL);
-		klass->parse_error_response (GDATA_SERVICE (self), GDATA_SERVICE_ERROR_WITH_INSERTION, status, message->reason_phrase, message->response_body->data,
-					     message->response_body->length, error);
+		klass->parse_error_response (GDATA_SERVICE (self), GDATA_OPERATION_UPDATE, status, message->reason_phrase,
+		                             message->response_body->data, message->response_body->length, error);
 		g_object_unref (message);
 		return NULL;
 	}
@@ -651,7 +641,7 @@ gdata_documents_service_move_document_to_folder (GDataDocumentsService *self, GD
 
 	/* Parse the XML; and update the document*/
 	new_document = GDATA_DOCUMENTS_ENTRY (gdata_parsable_new_from_xml (G_OBJECT_TYPE (document), message->response_body->data,
-									   message->response_body->length, error));
+	                                                                   message->response_body->length, error));
 	g_object_unref (message);
 
 	return new_document;
@@ -677,12 +667,11 @@ GDataDocumentsEntry *
 gdata_documents_service_remove_document_from_folder (GDataDocumentsService *self, GDataDocumentsEntry *document, GDataDocumentsFolder *folder,
 						     GCancellable *cancellable, GError **error)
 {
+	const gchar *folder_id, *document_id, *uri_template;
 	GDataServiceClass *klass;
-	gchar *uri;
-	const gchar *document_id, *folder_id;
 	SoupMessage *message;
-	GDataDocumentsEntry *new_document;
 	guint status;
+	gchar *uri;
 
 	g_return_val_if_fail (GDATA_IS_DOCUMENTS_SERVICE (self), NULL);
 	g_return_val_if_fail (GDATA_IS_DOCUMENTS_ENTRY (document), NULL);
@@ -691,7 +680,7 @@ gdata_documents_service_remove_document_from_folder (GDataDocumentsService *self
 
 	if (gdata_service_is_authenticated (GDATA_SERVICE (self)) == FALSE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
-				     _("You must be authenticated to move documents."));
+		                     _("You must be authenticated to move documents."));
 		return NULL;
 	}
 
@@ -702,14 +691,17 @@ gdata_documents_service_remove_document_from_folder (GDataDocumentsService *self
 	g_assert (document_id != NULL);
 
 	if (GDATA_IS_DOCUMENTS_PRESENTATION (document))
-		uri = g_strdup_printf ("http://docs.google.com/feeds/folders/private/full/folder%%3A%s/presentation%%3A%s", folder_id, document_id);
+		uri_template = "%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/presentation%%3A%s";
 	else if (GDATA_IS_DOCUMENTS_SPREADSHEET (document))
-		uri = g_strdup_printf ("http://docs.google.com/feeds/folders/private/full/folder%%3A%s/spreadsheet%%3A%s", folder_id, document_id);
+		uri_template = "%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/spreadsheet%%3A%s";
 	else if (GDATA_IS_DOCUMENTS_TEXT (document))
-		uri = g_strdup_printf ("http://docs.google.com/feeds/folders/private/full/folder%%3A%s/document%%3A%s", folder_id, document_id);
+		uri_template = "%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/document%%3A%s";
+	else if (GDATA_IS_DOCUMENTS_FOLDER (document))
+		uri_template = "%s://docs.google.com/feeds/folders/private/full/folder%%3A%s/folder%%3A%s";
 	else
 		g_assert_not_reached ();
 
+	uri = g_strdup_printf (uri_template, _gdata_service_get_scheme (), folder_id, document_id);
 	message = soup_message_new (SOUP_METHOD_DELETE, uri);
 	g_free (uri);
 
@@ -720,12 +712,6 @@ gdata_documents_service_remove_document_from_folder (GDataDocumentsService *self
 
 	soup_message_headers_append (message->request_headers, "If-Match", gdata_entry_get_etag (GDATA_ENTRY (document)));
 
-	/* Check for cancellation */
-	if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE) {
-		g_object_unref (message);
-		return NULL;
-	}
-
 	/* Send the message */
 	status = _gdata_service_send_message (GDATA_SERVICE (self), message, error);
 	if (status == SOUP_STATUS_NONE) {
@@ -742,21 +728,46 @@ gdata_documents_service_remove_document_from_folder (GDataDocumentsService *self
 	if (status != 200) {
 		/* Error */
 		g_assert (klass->parse_error_response != NULL);
-		klass->parse_error_response (GDATA_SERVICE (self), GDATA_SERVICE_ERROR_WITH_INSERTION, status, message->reason_phrase, message->response_body->data,
-					     message->response_body->length, error);
+		klass->parse_error_response (GDATA_SERVICE (self), GDATA_OPERATION_UPDATE, status, message->reason_phrase, message->response_body->data,
+		                             message->response_body->length, error);
 		g_object_unref (message);
 		return NULL;
 	}
 
-	/* Build the updated entry */
-	g_assert (message->response_body->data != NULL);
-
-	/* Parse the XML; and update the document*/
-	new_document = GDATA_DOCUMENTS_ENTRY (gdata_parsable_new_from_xml (G_OBJECT_TYPE (document), message->response_body->data,
-									   message->response_body->length, error));
 	g_object_unref (message);
 
-	return new_document;
+	/* Google's servers don't return an updated copy of the entry, so we have to query for it again.
+	 * See: http://code.google.com/p/gdata-issues/issues/detail?id=1380 */
+	return gdata_documents_service_query_single_document (self, G_OBJECT_TYPE (document), gdata_documents_entry_get_document_id (document),
+	                                                      cancellable, error);
+}
+
+/**
+ * gdata_documents_service_get_upload_uri:
+ * @folder: the #GDataDocumentsFolder into which to upload the document, or %NULL
+ *
+ * Gets the upload URI for documents for the service.
+ *
+ * If @folder is %NULL, the URI will be the one to upload documents to the "root" folder.
+ *
+ * Return value: the URI permitting the upload of documents to @folder, or %NULL; free with g_free()
+ *
+ * Since: 0.5.0
+ **/
+gchar *
+gdata_documents_service_get_upload_uri (GDataDocumentsFolder *folder)
+{
+	g_return_val_if_fail (folder == NULL || GDATA_IS_DOCUMENTS_FOLDER (folder), NULL);
+
+	/* If we have a folder, return the folder's upload URI */
+	if (folder != NULL) {
+		const gchar *folder_id = gdata_documents_entry_get_document_id (GDATA_DOCUMENTS_ENTRY (folder));
+		g_assert (folder_id != NULL);
+		return g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/folders/private/full/folder%3A", folder_id, NULL);
+	}
+
+	/* Otherwise return the default upload URI */
+	return g_strconcat (_gdata_service_get_scheme (), "://docs.google.com/feeds/documents/private/full", NULL);
 }
 
 GDataService *
