@@ -121,7 +121,7 @@ gdata_documents_spreadsheet_new (const gchar *id)
  * If @destination_file is a directory, then the file will be downloaded in this directory with the #GDataEntry:title with 
  * the apropriate extension as name.
  *
- * If there is an error getting the document, a %GDATA_SERVICE_ERROR_WITH_QUERY error will be returned.
+ * If there is an error getting the document, a %GDATA_SERVICE_ERROR_PROTOCOL_ERROR error will be returned.
  *
  * Return value: the document's data, or %NULL; unref with g_object_unref()
  *
@@ -192,8 +192,11 @@ gdata_documents_spreadsheet_get_download_uri (GDataDocumentsSpreadsheet *self, G
 
 	fmcmd = export_formats[export_format].fmcmd;
 
-	if (gid != -1)
-		return g_strdup_printf ("http://spreadsheets.google.com/feeds/download/spreadsheets/Export?key=%s&fmcmd=%s&gid=%d", document_id, fmcmd, gid);
-	else
-		return g_strdup_printf ("http://spreadsheets.google.com/feeds/download/spreadsheets/Export?key=%s&fmcmd=%s", document_id, fmcmd);
+	if (gid != -1) {
+		return g_strdup_printf ("%s://spreadsheets.google.com/feeds/download/spreadsheets/Export?key=%s&fmcmd=%s&gid=%d",
+		                        _gdata_service_get_scheme (), document_id, fmcmd, gid);
+	} else {
+		return g_strdup_printf ("%s://spreadsheets.google.com/feeds/download/spreadsheets/Export?key=%s&fmcmd=%s",
+		                        _gdata_service_get_scheme (), document_id, fmcmd);
+	}
 }
